import csv
from time import sleep
import pandas as pd
from selenium import webdriver
from selenium.webdriver import DesiredCapabilities
from links_scapper import YoutubeLinks
from video import Video

options = webdriver.ChromeOptions()
options.add_argument("--headless")


class VideoData(Video):
    """
    This class will iterate through the links stored in the
    json files created from the LinkScrapper class. It also inherits attributes
    functions from both of the Video and LinkScrapper class.

    Attributes:
        channel (str): this holds the value of the youtube channel video links will be scrapped from
        scroll_range (int): this specifies how far to scroll down the youtube page to grab links.
                            Note: 30 videos are captured per each scroll. When variable is set to 0,
                            30 videos are returned
        driver (str): contains the specified Chrome Driver path
    """

    def __init__(self, channel: str, scroll_range: int):
        self.driver = webdriver.Remote("http://127.0.0.1:4444/wd/hub", DesiredCapabilities.CHROME, options=options)
        self.links = YoutubeLinks(channel, scroll_range)
        self.channel = channel
        self.scroll_range = scroll_range
        self.yt_data = []

    def write_data_to_csv(self):
        df = pd.DataFrame.from_dict(self.yt_data, orient='columns')

        csv_file = open(
            './youtube_data.csv', 'w', encoding="UTF-8", newline="")
        writer = csv.writer(csv_file)
        writer.writerow(['title', 'description', 'hash_tags', 'views', 'comments'])
        df.to_csv('./youtube_data.csv')
        csv_file.close()

    def get_yt_data(self):
        """
        This function takes the json files containing links scrapped from the LinkScrapper,
        iterates over each link in the file, grabs the information described in the Video class
        and and stores them into a csv file.
        """
        count = 0
        for i, link in enumerate(self.links.extract_links()):
            print(f'I am collecting information for video {i} ')

            self.driver.get(link)
            data = {"title": [], "description": [], "loc_tags": [], "views": [], "comments": []}

            title = self.get_video_title()
            data['title'].append(title)

            desc = self.get_video_description()
            data['description'].append(desc)

            tags = self.get_hash_tag()
            data['loc_tags'].append(tags)

            views = self.get_no_views()
            data['views'].append(views)

            comm = self.get_video_comments()
            data['comments'].append([my_elem.text for my_elem in comm])

            self.yt_data.append(data)
            count += 1
            sleep(3)

        self.write_data_to_csv()
        self.driver.close()


if __name__ == '__main__':
    ls = VideoData('SkyNews', 0)
    ls.get_yt_data()



from selenium import webdriver
from selenium.webdriver import DesiredCapabilities
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait as Wait
from selenium.webdriver.support import expected_conditions as ec
import json
import re
import sys
from time import sleep

options = webdriver.ChromeOptions()
options.add_argument("--headless")


class YoutubeLinks:
    """
    This class is used to scrape links for youtube videos and save them in a json file

    class attributes:
        channel (str): this holds the value of the youtube channel video links will be scrapped from
        scroll_range (int): this specifies how far to scroll down the youtube page to grab each video links.
        driver (str): contains the specified Chrome Driver path
    """

    def __init__(self, channel: str, scroll_range: int):
        self.links = []
        self.channel = channel
        self.scroll_range = scroll_range
        self.driver = webdriver.Remote("http://127.0.0.1:4444/wd/hub", DesiredCapabilities.CHROME, options=options)


    @staticmethod
    def _is_channel_valid(x):
        x = x.lower()
        x = x.replace(' ', '')
        return x

    @property
    def channel(self):
        return self.__channel

    @channel.setter
    def channel(self, channel):
        if re.fullmatch('[a-zA-Z\s]+', channel):
            self.__channel = self._is_channel_valid(channel)
        else:
            sys.stderr.write('Not a valid channel')
            raise ValueError('Not a valid channel')

    def accept_cookies(self) -> None:
        try:
            cookies = self.driver.find_element_by_xpath(
                '//*[@id="yDmH0d"]/c-wiz/div/div/div/div[2]/div[1]/div[4]/form/div[1]/div/button/span')
            cookies.click()
        except:
            pass

    def page_scroll(self) -> None:
        for i in range(0, self.scroll_range):
            self.driver.execute_script(
                "window.scrollTo(0,Math.max(document.documentElement.scrollHeight,document.body.scrollHeight,"
                "document.documentElement.clientHeight))")
            sleep(2)

    def extract_links(self):
        """
        This function performs the links scrapping using the methods above
        """
        url = f'https://www.youtube.com/c/{self.channel}/videos'
        self.driver.get(url)
        self.accept_cookies()
        sleep(2)
        self.page_scroll()
        sleep(2)
        videos_list = Wait(self.driver, 30).until(
            ec.presence_of_all_elements_located((By.XPATH, "//*[@id='video-title']")))
        self.links = []
        for item in videos_list:
            self.links.append(item.get_attribute('href'))
        print(f' I found {len(self.links)} videos')
        self.write_to_json()
        self.driver.close()
        return self.links

    def write_to_json(self):
        with open('./links.json', 'w') as f:
            json.dump(self.links, f, indent=2)


if __name__ == '__main__':
    ls = YoutubeLinks('skynews', 5)
    ls.extract_links()



from selenium import webdriver
from selenium.webdriver.support.ui import WebDriverWait as Wait
from selenium.webdriver.support import expected_conditions as ec
from selenium.webdriver.common.by import By
from time import sleep


class Video:
    """
    This class is used to provide the video information that will be scrapped by
    Scrapper.

    class attributes:
        driver (str): contains the specified Chrome Driver path

    """

    def __init__(self):
        self.driver = webdriver.Chrome(executable_path=driver_path, options=options)

    def get_video_title(self):
        try:
            v_title = Wait(self.driver, 5).until(
                ec.presence_of_element_located((By.XPATH, '//*[@id="container"]/h1/yt-formatted-string'))).text
        except:
            v_title = ""
        return v_title

    def get_video_description(self):
        try:
            v_description = Wait(self.driver, 5).until(ec.presence_of_element_located(
                (By.CSS_SELECTOR, "#description > yt-formatted-string > span:nth-child(1)"))).text.split('\n')
        except:
            v_description = ""
        return v_description

    def get_hash_tag(self):
        try:
            v_hash_tags = Wait(self.driver, 5).until(
                ec.presence_of_element_located((By.XPATH, '//*[@id="container"]/yt-formatted-string/a'))).text
        except:
            v_hash_tags = ""
        return v_hash_tags

    def get_no_views(self):
        try:
            v_views = self.driver.find_element_by_xpath(
                '//span[@class="view-count style-scope ytd-video-view-count-renderer"]').text
        except:
            v_views = ""
        return v_views

    def get_video_comments(self):
        """
        This function is used to grab video comments
        """
        sleep(2)
        self.driver.execute_script('window.scrollTo(0,100);')
        sleep(2)
        for i in range(0, 4):
            self.driver.execute_script(
                "window.scrollTo(0,Math.max(document.documentElement.scrollHeight,document.body.scrollHeight,"
                "document.documentElement.clientHeight))")
            sleep(2)
        try:
            v_comments = Wait(self.driver, 15).until(
                ec.presence_of_all_elements_located((By.XPATH, '//*[@id="content-text"]')))
        except:
            v_comments = ""
        return v_comments



from video_data import VideoData

print('This is a Web_scrapper package for Youtube')
print('Input the information required and let me do the rest :)')
channel = (input('Youtube channel: '))
scroll_range = int(input('Number of scrolls: '))
scraper = VideoData(channel, scroll_range)
data = scraper.get_yt_data()
print(f"Scrapping completed\nSee csv file for data.\nThanks for using YouTube scrapper")



import psycopg2
import pandas as pd
from sqlalchemy import create_engine


class Dbload:
    def __init__(self, password, host):
        self.password = password
        self.host = host
        self.tablename = 'YoutubeData'
        self.df = pd.read_csv("./youtube_data.csv")
        self.engine = create_engine(f"postgresql+psycopg2://postgres:{password}@{host}:5432/postgres")
        self.conn = psycopg2.connect(f"dbname=postgres user=postgres password={password} host=3.21.144.229 port=5432")

    def create_table(self):
        cursor = self.conn.cursor()
        cursor.execute("DROP TABLE IF EXISTS YoutubeData")
        cursor.execute(
            "CREATE TABLE YoutubeData (title VARCHAR(255), description VARCHAR(500), hash_tags VARCHAR(30), "
            "views VARCHAR(30), comments VARCHAR(20000))")
        self.conn.commit()

    def save_to_database(self):
        self.create_table()
        # Do not store into postgres with an index column
        self.df.to_sql(self.tablename, self.engine, if_exists='replace', index=False)
        print(f"Created table: {self.tablename}")

    def read_table(self, tbl):
        self.engine.execute(f'''SELECT * FROM {tbl}
            LIMIT 20''').fetchall()


if __name__ == '__main__':
    sv = Dbload(input('Enter password: '), input('Enter host: '))
    sv.save_to_database()
